#!/bin/bash 

####################################################################################
### Завершаем работу скрипта при ошибках
####################################################################################

set -e

####################################################################################
### Задаём переменные
####################################################################################

# Переменные, настраиваемые пользователем, храним отдельно. 
source /app/asr_billing/cfg/sorm3-mfi/config

####################################################################################
### Основные функции
####################################################################################

# Функция добавляет служебный реквизит с типами документов 
prepare_references(){
    # Проверяем есть ли реквизит
    does_attr_exist=$(sqlexec "set heading off; select count(*) from user_attributes where attribute_id=-332891" | sed 's/ *//g; /^$/d')

    if [[ $does_attr_exist == 0 ]]; then 
        # Если нет - добавляем
        echo "Нет реквизита \"Тип документа\" с ID -332891, добавляем"
        curl -XPOST 'http://169.254.80.82:8082/rest_api/v2/UserAttributes/' -d 'method1=objects.create&arg1={"attribute_id":-332891,"name": "Тип документа","description": "Документ удостоверяющий личность","type_id": 3,"type_attr": "Неизвестно,Военный билет,Временное удостоверение личности гражданина РФ,Иностранный паспорт,Паспорт гражданина РФ,Паспорт гражданина СССР,Свидетельство о регистрации ходатайства иммигранта,Свидетельство о рождении,Справка об освобождении из мест лишения свободы,Удостоверение беженца в РФ,Удостоверение личности офицера,Вид на жительство в РФ иностранных граждан,Загранпаспорт гражданина СССР,Загранпаспорт гражданина РФ,Паспорт Минморфлота,Дипломатический паспорт гражданина РФ,Удостоверение личности,Свидетельство о рождении выданное уполномоченным органом иностранного государства,Паспорт моряка,Иные документы выданные МВД","framework_form_groups_id": null}'
    else 
        # Если есть - пишем об этом и идём дальше
        echo "Реквизит \"Тип документа\" с ID -332891 уже создан"
    fi 
}

# Функция добавляет реквизит с типом документов физлица у которых его ещё нет
prepare_abonents(){
    declare -a abonents 

    # Получаем список абонентов без реквизита "Тип документа"
    abonents=( $(sqlexec -l "
    select distinct 
        a.id 
    from 
        abonents a 
    left join 
        attribute_values av 
        on a.id=av.abonent_id 
        and attribute_id=-332891 
    where 
        coalesce(a.company,0)=0 
        and coalesce(a.category_id,1)=1 
        and a.is_folder=0 
        and a.id>0 and (av.id is null)" | awk '$2{print $2}' | tr '\n' ' ' ) )

    # Пишем количество абонентов без реквизита для лога
    echo  "Абонентов без типа документа: ${#abonents[*]}"

    for abon_id in ${abonents[*]}; do 
        # Пишем ID абонента для лога
        echo "Добавляем абоненту $abon_id реквизит \"Тип документа\" со значением по-умолчанию \"Паспорт гражданина РФ\""
        curl -XPOST 'http://169.254.80.82:8082/rest_api/v2/AttributeValues/' --data 'method1=objects.create&arg1={"abonent_id":'$abon_id',"attribute_id":"-332891","attribute_value":"Паспорт гражданина РФ"}'
    done
}

# Функция добавляет в планировщик ОС задание выгрузки в СОРМ 
install_cron(){
    if [[ ! -L /etc/cron.d/sorm3-mfi-upload.sh ]]; then
        echo "Настраиваем cron"
        ln -s /app/asr_billing/cfg/sorm3-mfi/etc-host/cron.d/sorm3-mfi-upload.sh /etc/cron.d/sorm3-mfi-upload.sh
        service crond restart &
    else
        echo "cron уже настроен: файл или символьная ссылка /etc/cron.d/sorm3-mfi-upload.sh существуют"
    fi 
}

# Функция настраивает ротировние лога выгрузки в СОРМ 
install_logrotate(){
    if [[ ! -L /app/asr_billing/etc/logrotate.d/sorm3 ]]; then
        echo "Настраиваем logrotate"
        ln -s /app/asr_billing/cfg/sorm3-mfi/etc-chroot/logrotate.d/sorm3 /app/asr_billing/etc/logrotate.d/sorm3
        chroot /app/asr_billing/ service rsyslog restart &
    else
        echo "logrotate уже настроен: файл или символьная ссылка /app/asr_billing/etc/logrotate.d/sorm3 существуют"
    fi 
}

prepare_payment_report(){

    ##
    ## В этой функции много "прибито гвоздями", но подойдёт для большинства операторов
    ## Если что-то менять - то нужно делать очень аккуратно, сверившись с документацией и подержкой
    ##

    echo "Делаем undefined_payment.sql и pay_type.csv"

    # Определяем пути к файлам
    u_payment_template="$sql_dir/undefined_payment_template.sql"
    u_payment_tmp="$sql_dir/undefined_payment_temp.sql"
    u_payment="$sql_dir/undefined_payment.sql"
    p_types_csv="$data_dir/pay_type.csv_raw"

    # Удаляем все файлы что будем переделывать
    rm -f $u_payment_tmp $u_payment $p_types_csv 
    
    # Получаем список операторов связи
    isp_list=( $(sqlexec -l "set heading off; select id from abonents where is_folder=0 and deleted=0 and category_id=2" | awk '$2{print $2}' | tr '\n' ' ') )

    # Получаем список платёжных систем 
    pay_systems=( $(grep -shir "paywork.*new" /app/asr_fiscal/usr/local/www/htdocs/*  | sed 's/   *//g; s/this->//g ;s/k=n/k = n/g; s/,/, /g' | awk '{print $5}' | sed -e "/^$/d; s/);//g; s/'//g" | sort | uniq) )
    
    # Заполняем временный файл для создания sql из шаблона и финальный csv pay_types
    p_system_num=1
    for pay_system in ${pay_systems[*]}; do 
        # Отправляем строку во временный файл для формирования case в sql
        echo -e "\t\twhen fo.operator_name = '$pay_system' then '$p_system_num'" >> $u_payment_tmp
        # Отправляем строку в csv типов платежей для последющей выгрузки, по каждому оператору связи
        for isp in ${isp_list[*]}; do
            echo -e "$p_system_num;31.12.1970 23:59:59;31.12.2024 23:59:59;$pay_system;$isp" >> $p_types_csv
        done 
        ((p_system_num++))
    done
    # Добавляем для case последнюю строку с "неопределённыи типом"
    echo -e "\t\twhen fo.operator_name is null then '$p_system_num'\n\t\telse '$p_system_num'" >> $u_payment_tmp
    # Отправляем строку в csv типов платежей "неопределённый тип", по каждому оператору связи
    for isp in ${isp_list[*]}; do
        echo -e "$p_system_num;31.12.1970 23:59:59;31.12.2024 23:59:59;Поступление наличных;$isp" >> $p_types_csv
    done 

    # Собираем sql файл выгрузок платежей по частям
    head -n 3 $u_payment_template >> $u_payment
    cat $u_payment_tmp >> $u_payment
    tail $u_payment_template -n+5 >> $u_payment
    # Удаляем временный файл 
    rm -f $u_payment_tmp

    # Выгружаем актуальный справочник типов платежей
    $scrits_dir/upload.sh pay_type
}

####################################################################################
### Выполнение скрипта
####################################################################################

# Первым агрументом можно выполнить конкретное действие
# Если запустили без агругментов - делаем всё
case ${1:-not_set} in 
    not_set ) 
        prepare_references
        prepare_abonents
        install_cron
        install_logrotate
        prepare_payment_report;;
    *       ) 
        $1;;
esac
